; -------------------------------------
; nibble to hex
;
; convert one nibble to ascii hex char
;
; params: nibble in A
;         c=1 high nibble, 
;         c=0 low nibble
; affects: A
; return: char in A
; -------------------------------------

HexCode bcc Hex1
	lsr	    ; get hi nibble
	lsr
	lsr
	lsr
Hex1	and #$0f
	cmp #$0a    ; is it a char?
        bmi digit   ; no, then branch
        ; convert to char
        clc
        adc #$37
        rts
        ; convert to digit
digit   ora #$30
        rts        

; -------------------------------------
; TestL - Test letter
;
; params: char in A
; affects: none
; return: c = 1 for letter
;         c = 0 for no letter
; -------------------------------------
TestL   cmp #$41    ; smaller then A?
        bcs TL1     ; no -> go on
        rts         ; return no letter
        
TL1     cmp #$5b    ; smaller then Z+1?
        bcs TL2     ; no -> go on
        sec         ; return letter
        rts
        
TL2     cmp #$61    ; smaller then a?
        bcs TL3     ; no -> go on
        rts         ; return no letter
        
TL3     cmp #$7b    ; smaller then z+1?        
        bcs TL4     ; no -> go on
        sec         ; return letter
        rts
        
TL4     clc         ; return no letter
        rts        

; -------------------------------------
; TestD - test digit
;
; params: char in A
; affects: none
; return: c = 1 for digit
;         c = 0 for no digit
; -------------------------------------
TestD   cmp #$30    ; smaller then 0?
        bcs TD1     ; no -> go on
        rts         ; return no digit

TD1     cmp #$3a    ; smaller then 9+1?
        bcs TD2     ; no -> go on
        sec         ; return digit
        rts

TD2     clc         ; return no digit
        rts

; -------------------------------------
; IntStr - integer to string
;
; params: lo/hi byte in xVector
; affects: A
; uses: xVector, yVector
; return: string in DecNum, terminated
;         with $00
;
; Makes a four-digit number.
; xVector will be destroyed.
; Leading zeros are $80 char and
; thus ignored by PrnChr/CPrnStr.
; -------------------------------------
IntStr  txa	      ; save X
	pha
	; -- loop digits --
	ldx #$00
ISdig	lda #$30-1    ; prepare digit
	sta DecNum,x
	lda IStabL,x  ; power of 10
	sta yVector
	lda IStabH,x
	sta yVector+1
ISdec	inc DecNum,x  ; inc digit
	jsr sub16     ; dec value
	bcs ISdec     ; ok? -> again
	jsr add16     ; else-> inc value
	inx
	cpx #$04      ; next digit 1er?
	bne ISdig     ; no -> next digit
	; -- remaining 8 bit number --
	lda #$30-1    ; "0" - 1
	adc xVector
        sta DecNum+4  ; -> add val + c
	; -- terminate string --
        lda #$00
        sta DecNum+5
        ; -- erase leading zeros --
	ldx #$00
IS11	lda DecNum,x  ; digit = 0?
	cmp #$30
	bne ISend     ; no -> finished
	lda #$80      ; yes-> erase
	sta DecNum,x
	inx
	cpx #$04      ; next digit 1er?
	beq ISend     ; -> finished
	jmp IS11
ISend	pla	      ; restore X
	tax
        rts

IStabL .byt <10000, <1000, <100, <10
IStabH .byt >10000, >1000, >100, >10

; -------------------------------------
; sub16 - subtract 16 bit numbers
;
; params: xVector first number
;         yVector second bumber
; affects: A
; returns: xVector = xVector - yVector
;          c = 0 underrun
;          c = 1 ok
; -------------------------------------
sub16   sec
	lda xVector
	sbc yVector
	sta xVector
	lda xVector+1
	sbc yVector+1
	sta xVector+1
	rts

; -------------------------------------
; add16 - add 16 bit numbers
;
; params: xVector first number
;         yVector second bumber
; affects: A
; returns: xVector = xVector + yVector
;          c = 0 ok
;          c = 1 overvlow
; -------------------------------------
add16   clc
	lda xVector
	adc yVector
	sta xVector
	lda xVector+1
	adc yVector+1
	sta xVector+1
	rts

; -------------------------------------
; addDecDig - add decimal digit
;
; multiply A * 10, add Y
;
; param: present number in A
;        new digit in Y (may be ASCII digit)
; affects: A
; return: 10 times the number in A + Y
;         c = 1 overflow occured,
;	      number invalid
;         c = 0 no overflow
; -------------------------------------
addDecDig
        ; --- inc value ---
        ; old value * 10
        ; 10a = ( 4a + a ) * 2
	sta mul10buf
        clc
        asl           ; ( 4a
        bcs aDDigE
        asl
        bcs aDDigE
        adc mul10buf  ; + a )
        bcs aDDigE
        asl           ; *2 
        bcs aDDigE       
	sta mul10buf
	; --- add Y ---
	tya
	and #$0f      ; digit to val
	adc mul10buf
	
aDDigE	rts

; -------------------------------------
; NumRange - check range of number
;
; params: A number to check
;         X min value
;         Y max value
; uses: xVector
; return: c = 0  not in range
;         c = 1  in range
; -------------------------------------
NumRange
	; compare to min
	stx xVector
	cmp xVector
	bcc NumRend	; A < min: c=0
	; compare to max
	sta xVector
	cpy xVector	; max < A: c=0

NumRend rts

; -------------------------------------
; strcpy - copy null terminated string
;
; affects: A, Y
; params: ptr to source in xVector
;         ptr to dest in yVector
; -------------------------------------
strcpy	ldy #$00
strcpy0	lda (xVector),y
	sta (yVector),y
	beq strcpy1
	iny
	jmp strcpy0
strcpy1	rts


; vim: set syntax=asm6502 :
