; *************************************
; *                                   *
; * 9600 baud with user port          *
; *                                   *
; * taken from Daniel Dallmanns       *
; * sample implementation, adapted    *
; * for CaTer                         *
; *                                   *
; * This file is part of CaTer.       *
; * (C) Lars Stollenwerk 2001 - 2003  *
; *                                   *
; * CaTer is provided under the terms *
; * of GNU General Public License.    *
; *                                   *
; * For more information see the      *
; * README file.                      *
; *                                   *
; *************************************

;; rewritten (based on LUnix' 
;;            getty code)
;;  UP9600
;;    (universal) device dirver 
;;    for RS232 userport interface with
;;    special wiring.

;; Nov 23 1997 by Daniel Dallmann


; *************************************
; *                                   *
; * provided functions                *
; *                                   *
; *************************************

; --- install ---
; install and (probe for) UP9600 
; (c=error)        

; --- enable ---
; (re-)enable interface

; --- disable ---
; disable interface 
; (eg. for floppy accesses)

; --- uninstall ---
; disable interface and restore NMI

;; rsout and rsin both modify A 
;; and X register
        
; --- rsout ---
; put byte to RS232 (blocking)

; --- rsin ---
; read byte from RS232 (c=try_again)


; *************************************
; *                                   *
; *      constant declaration         *
; *                                   *
; *************************************
                
; lowest byte of system's 
; jiffie counter
jiffies = $a2
        
; (must incease jiffie-counter !)
;original_irq = $ea31  
original_irq = IRQ

original_nmi = $fe47

nmi_vect = $0318
irq_vect = $0314



; -------------------------------------
; variables 
;
; -------------------------------------


; copy of $a2=jiffies to detect timeouts
stime = $8a 

outstat = $8b

; write-pointer into send buffer     
wr_sptr = $8c

; read-pointer into send buffer
rd_sptr = $8d

; write-pointer into receive buffer
wr_rptr = $8e

; read-pointer into receive buffer
rd_rptr = $8f

; 128 bytes
revtab = $0d00

; 256 bytes
recbuf = $0e00

; 256 bytes
sndbuf = $0f00

                
; *************************************
; *                                   *
; *           routines                *
; *                                   *
; *************************************

; -------------------------------------
; NMI part
;
; nmi_startbit and nmi_bytrdy must be
; in the same code page !!!
; -------------------------------------

nmi_startbit
        pha
        bit  $dd0d               ; check bit 7 (startbit ?)
        bpl  nmi1                ; no startbit received, then skip
        
        lda  #$13
        sta  $dd0f              ; start timer B (forced reload, signal at PB7)
        sta  $dd0d              ; disable timer and FLAG interrupts
        lda  #<nmi_bytrdy       ; on next NMI call nmi_bytrdy
        sta  nmi_vect           ; (triggered by SDR full)

nmi1    
        pla                     ; ignore, if NMI was triggered by RESTORE-key
        rti

nmi_bytrdy
        pha
        bit  $dd0d              ; check bit 7 (SDR full ?)
        bpl  nmi1               ; SDR not full, then skip (eg. RESTORE-key)
        
        lda  #$92
        sta  $dd0f              ; stop timer B (keep signalling at PB7!)
        sta  $dd0d              ; enable FLAG (and timer) interrupts
        lda  #<nmi_startbit     ; on next NMI call nmi_startbit
        sta  nmi_vect           ; (triggered by a startbit)
        txa
        pha
        lda  $dd0c              ; read SDR (bit0=databit7,...,bit7=databit0)
        cmp  #$80               ; move bit7 into carry-flag
        and  #$7f
        tax
        lda  revtab,x           ; read databits 1-7 from lookup table
        adc  #$00               ; add databit0
        ldx  wr_rptr            ; and write it into the receive buffer
        sta  recbuf,x
        inx
        stx  wr_rptr
        sec
        txa
        sbc  rd_rptr
        cmp  #$c8
        bcc  nmi2
        lda  $dd01              ; more than 200 bytes in the receive buffer
        and  #$fd               ; then disbale RTS
        sta  $dd01
nmi2    pla
        tax
        pla
        rti

; -------------------------------------
; IRQ part
;
; -------------------------------------

new_irq
        lda  $dc0d              ; read IRQ-mask
        lsr   
        lsr                     ; move bit1 into carry-flag (timer B - flag)
        and  #$02               ; test bit3 (SDR - flag)
        beq  nirq1              ; SDR not empty, then skip the first part
        ldx  outstat
        beq  nirq1              ; skip, if we're not waiting for an empty SDR
        dex
        stx  outstat
        bne  nirq1              ; skip, if we're not waiting for an empty SDR
        
        php
        jsr  send_nxtbyt        ; send the next databyte
        plp
        
nirq1   bcs  nirq2              ; skip if there was no timer-B-underflow
        jmp  $ea81              ; return from IRQ
        
        ; keyscan IRQ

nirq2   sec
        ; (a lost SDR-empty interrupt, would
        ; totally lock up the sender. So i've added
        ; a timeout)        
        lda  jiffies
        cmp  stime              ; timeout as short as possible
        beq  nirq3              ; for smooth keyboard feeling
        ;;sbc  stime            ; originally:
        ;;cmp  #$10             ; (timeout after 16/64 = 0.25 seconds)
        ;;bcc  nirq3
        jsr  send_nxtbyt        ; send the next databyte
nirq3   
        jmp  original_irq



; -------------------------------------
; send next byte from buffer
;
; -------------------------------------
        
send_nxtbyt
        lda  jiffies            ; remember jiffie counter for detecting
        sta  stime              ; timeouts
        lda  $dd01              ; check CTS line from RS232 interface
        and  #$40
        beq  sn1                ; skip (because CTS is inactive)
        ldx  rd_sptr
        cpx  wr_sptr
        beq  sn1                ; skip (because buffer is empty)
        lda  sndbuf,x
        inx
        stx  rd_sptr
        cmp  #$80               ; move bit7 into carry-flag
        and  #$7f               ; get bits 1-7 from lookup table
        tax
        lda  revtab,x
        adc  #$00               ; add bit0
        lsr   
        sta  $dc0c              ; send startbit (=0) and the first 7 databits
        lda  #$02               ; (2 IRQs per byte sent)
        sta  outstat
        ror  
        ora  #$7f               ; then send databit7 and 7 stopbits (=1)
        sta  $dc0c              ; (and wait for 2 SDR-empty IRQs or a timeout
sn1     rts                     ; before sending the next databyte)
        

; -------------------------------------
; get byte from serial interface
;
; -------------------------------------
        
rsin    ldx  rd_rptr
        cpx  wr_rptr
        beq  rsin2              ; skip (empty buffer, return with carry set)
        lda  recbuf,x
        inx
        stx  rd_rptr
        pha
        txa
        sec
        sbc  wr_rptr
        cmp  #256-50
        bcc  rsin1
        lda  #$02               ; enable RTS if there are less than 50 bytes
        ora  $dd01              ; in the receive buffer
        sta  $dd01
        clc
rsin1   pla
rsin2   rts


; -------------------------------------
; rsout - put byte to serial interface
;
; param: byte in A
; affects: A, X
; -------------------------------------
rsout   ldx  wr_sptr
        sta  sndbuf,x
        inx
rsout1  cpx  rd_sptr            ; wait for free slot in the send buffer
        beq  rsout1
        stx  wr_sptr
        lda  outstat
        bne  rsout2
        lda  jiffies
        eor  #$80
        sta  stime              ; force timeout on next IRQ
rsout2  rts


; -------------------------------------
; install (and probe for) serial interface
; return with carry set if there was an error
;
; -------------------------------------

inst_err 
        cli
        sec
        rts
        
install  
        sei
        lda  irq_vect
        cmp  #<original_irq
        bne  inst_err           ; IRQ-vector already changed 
        lda  irq_vect+1
        cmp  #>original_irq
        bne  inst_err           ; IRQ-vector already changed
        lda  nmi_vect
        cmp  #<original_nmi
        bne  inst_err           ; NMI-vector already changed 
        lda  nmi_vect+1
        cmp  #>original_nmi
        bne  inst_err           ; NMI-vector already changed

        ldy  #$00
        sty  wr_sptr
        sty  rd_sptr
        sty  wr_rptr
        sty  rd_rptr
        
        ;; probe for RS232 interface

        cli
        lda  #$7f
        sta  $dd0d              ; disable all NMIs
        lda  #$80
        sta  $dd03              ; PB7 used as output
        sta  $dd0e              ; stop timerA
        sta  $dd0f              ; stop timerB
        bit  $dd0d              ; clear pending interrupts
        ;; for debugging
        ;jmp noprobe ; debug
        ldx  #$08
probe1  stx  $dd01              ; toggle TXD
        sta  $dd01              ; and look if it triggers an
        dex                     ; shift-register interrupt
        bne  probe1
        lda  $dd0d              ; check for bit3 (SDR-flag)
        and  #$08
        beq  inst_err           ; no interface detected

;; for debugging
noprobe

        ;; generate lookup table
        
        ldx  #$00
look1   stx  outstat            ; outstat used as temporary variable
        ldy  #$08
look2   asl  outstat
        ror 
        dey
        bne  look2
        sta  revtab,x
        inx
        bpl  look1


; -------------------------------------
; enable serial interface (IRQ+NMI)
        
enable  sei
        
        ldx  #<new_irq          ; install new IRQ-handler
        ldy  #>new_irq
        stx  irq_vect
        sty  irq_vect+1
        
        ldx  #<nmi_startbit     ; install new NMI-handler
        ldy  #>nmi_startbit
        stx  nmi_vect
        sty  nmi_vect+1
        
        ldx  $02a6              ; PAL or NTSC version ?
        lda  ilotab,x           ; (keyscan interrupt once every 1/64 second)
        sta  $dc06              ; (sorry this will break code, that uses
        lda  ihitab,x           ; the ti$ - variable)
        sta  $dc07              ; start value for timer B (of CIA1)
        txa
        asl
        
        eor  #$33               ; ** time constant for sender **
        ldx  #$00               ; 51 or 55 depending on PAL/NTSC version
        sta  $dc04              ; start value for timerA (of CIA1)
        stx  $dc05              ; (time is around 1/(2*baudrate) )
        
        asl                     ; ** time constant for receiver **
        ora  #$01               ; 103 or 111 depending on PAL/NTSC version
        sta  $dd06              ; start value for timerB (of CIA2)
        stx  $dd07              ; (time is around 1/baudrate )
        
        lda  #$41               ; start timerA of CIA1, SP1 used as output
        sta  $dc0e              ; generates the sender's bit clock
        lda  #$01
        sta  outstat
        sta  $dc0d              ; disable timerA (CIA1) interrupt
        sta  $dc0f              ; start timerB of CIA1 (generates keyscan IRQ)
        lda  #$92               ; stop timerB of CIA2 (enable signal at PB7)
        sta  $dd0f
        lda  #$98
        bit  $dd0d              ; clear pending NMIs
        sta  $dd0d              ; enable NMI (SDR and FLAG) (CIA2)
        lda  #$8a
        sta  $dc0d              ; enable IRQ (timerB and SDR) (CIA1)
        lda  #$82               ; PB1 and PB7 as output
        sta  $dd03
        lda  #$ff
        sta  $dd01              ; PB0-7 default to 1
        sta  $dc0c              ; SP1 defaults to 1
        sec
        lda  wr_rptr
        sbc  rd_rptr
        cmp  #$c8
        bcs  enable1            ; don't enable RTS if rec-buffer is full
        lda  #$02               ; enable RTS 
        ora  $dd01              
        sta  $dd01        
enable1 cli
        rts

;; table of timer values for PAL and NTSC version
        
ilotab
        .byt $95
        .byt $25
ihitab 
        .byt $42
        .byt $40       



; -------------------------------------
; disable serial interface
;
; affects: A, X
; -------------------------------------

disable  
        lda  $dd01              ; disable RTS
        and  #$fd
        sta  $dd01

;; If we do that, two CaTers can hang up each other if they both clear RTS
;; and want to access the floppy.
;disabl1 lda  wr_sptr            ; wait for empty 
;        cmp  rd_sptr            ; send buffer
;        bne  disabl1
        
        sei
        jsr $eeb3               ; wait 2 ms, maybe we receive something
        jsr $eeb3               ; and therefore need NMI and CIA

        lda  #$7f
        sta  $dd0d              ; disable all CIA interrupts
        sta  $dc0d
        lda  #$41               ; quick (and dirty) hack to switch back
        sta  $dc05              ; to the default CIA1 configuration
        lda  #$81
        sta  $dc0d              ; enable timer1 (this is default)

        lda  #<original_irq     ; restore old IRQ-handler
        sta  irq_vect
        lda  #>original_irq
        sta  irq_vect+1

        lda  #<NoNmi            ; do nothing in NMI
        sta  nmi_vect
        lda  #>NoNmi
        sta  nmi_vect+1
        cli
        rts

; do absolute nothing when device is disabled
NoNmi   rti


; -------------------------------------
; uninstall interface
;
; -------------------------------------
        
uninstall
        jsr  disable
        lda  #<original_nmi     ; restore old NMI-handler
        sta  nmi_vect
        lda  #>original_nmi
        sta  nmi_vect+1
        rts

; vim: set syntax=asm6502 :
